# -*- coding: utf-8 -*-
from django import forms
from django.template.defaultfilters import filesizeformat
from django.forms.utils import flatatt
from django.utils.safestring import mark_safe
from django.utils.html import escape, escapejs, conditional_escape
from django.conf import settings
from django.core.exceptions import ValidationError
from itertools import chain
from datetime import date

from bbcode.forms import BBCodeModelForm
from frappe.models import (
    Annee,
    Filiere,
    Matiere,
    Tag,
    Document,
    Commentaire,
    TAILLE_MAX_NOM_FICHIER,
    DOSSIER_STOCKAGE_FICHIERS,
)


class TagWidget(forms.CheckboxSelectMultiple):
    """ Widget pour les tags, basé sur les sources de CheckboxSelectMultiple """

    def render(self, name, value, attrs=None, choices=(), renderer=None):
        if value is None:
            value = []
        has_id = attrs and "id" in attrs
        final_attrs = self.build_attrs(attrs)
        output = []
        str_values = {v for v in value}  # normalize

        for i, (option_value, option_label) in enumerate(chain(self.choices, choices)):
            # If an ID attribute was given, add a numeric index as a suffix,
            # so that the checkboxes don't all have the same ID attribute.
            if has_id:
                final_attrs = dict(final_attrs, id=f"{attrs['id']}_{i}")
                label_for = f''' for="{final_attrs['id']}"'''
            else:
                label_for = ""

            cb = forms.CheckboxInput(final_attrs, check_test=lambda choice: choice.value in str_values)
            option_value = option_value
            rendered_cb = cb.render(name, option_value)
            option_label = conditional_escape(option_label)
            output.append(f'<label{label_for} class="label-tag">{rendered_cb} {option_label}</label>')
        return mark_safe("<div>" + "\n".join(output) + "</div>")


class TagField(forms.ModelMultipleChoiceField):
    def label_from_instance(self, tag):
        return mark_safe(
            f'<span class="badge text-white" style="background-color: {tag.color}; margin-right: 25px">{escape(tag.nom)}</span>'
        )


class MatiereWidget(forms.Select):
    class Media:
        js = ("frappe/matiere_widget.js",)

    def render(self, name, value, attrs=None, renderer=None):
        # Get the first non-default value
        first_value = None
        for e in self.choices:
            if e[0] != "":
                first_value = Matiere.objects.get(id=e[0].value)
                break
        self.ecole = first_value.filiere.annee.ecole
        self.annees = Annee.objects.filter(ecole=self.ecole).order_by("nom")
        self.filieres = Filiere.objects.filter(annee__ecole=self.ecole).select_related("annee").order_by("nom")

        if value is None:
            value = first_value
        else:
            value = Matiere.objects.select_related("filiere__annee__ecole", "filiere__annee", "filiere").get(
                filiere__annee__ecole=self.ecole, pk=value
            )

        # <script>
        output = ['<script type="text/javascript">']
        output.append("$(function () {")
        output.append("\tsetup_matiere_widget([")
        for filiere in self.filieres:
            output.append(f'\t\t{{"pk": {filiere.pk}, "annee": {filiere.annee.pk}, "nom": "{escapejs(filiere.nom)}"}},')
        output.append("\t], [")

        list_select = []
        for e in self.choices:
            if e[0] != "":
                matiere = Matiere.objects.get(id=e[0].value)
                output.append(
                    f'\t\t{{"pk": {matiere.pk}, "filiere": {matiere.filiere.pk}, "nom": "{escapejs(matiere.nom)}"}},'
                )
                if matiere.filiere == value.filiere:
                    list_select.append(matiere)
        output.append(f"""\t], $("#{attrs['id']}_annee"), $("#{attrs['id']}_filiere"), $("#{attrs['id']}"));""")
        output.append("});")
        output.append("</script>")

        # <select> pour l'année
        output.append(f"""<select id="{attrs['id']}_annee">""")
        output.append(self.render_options(self.annees, value.filiere.annee, lambda a: a.nom))
        output.append("</select>")

        # <select> pour la filière
        output.append(f"""<select id="{attrs['id']}_filiere">""")
        output.append(
            self.render_options(
                self.filieres.filter(annee=value.filiere.annee),
                value.filiere,
                lambda f: f.nom,
            )
        )
        output.append("</select>")

        # <select> pour la matière
        final_attrs = self.build_attrs(attrs)
        output.append(f'<select name="matiere" {flatatt(final_attrs)}>')
        output.append(self.render_options(list_select, value, lambda m: m.nom))
        output.append("</select>")
        return mark_safe("\n".join(output))

    def render_options(self, choices, selected_choice, label_fn):
        output = []
        for choice in choices:
            selected_html = ' selected="selected"' if choice == selected_choice else ""
            output.append(
                f'\t<option value="{choice.pk}"{selected_html}>{conditional_escape(label_fn(choice))}</option>'
            )

        return "\n".join(output)


class DocumentFileField(forms.FileField):
    def clean(self, *args, **kwargs):
        cleaned = super(DocumentFileField, self).clean(*args, **kwargs)

        if cleaned and cleaned.size > settings.FRAPPE_MAX_UPLOAD_SIZE:
            raise forms.ValidationError(
                "Fichier trop gros (taille maximum : {}).".format(filesizeformat(settings.FRAPPE_MAX_UPLOAD_SIZE))
            )

        return cleaned


class CreateDocumentForm(BBCodeModelForm):
    class Meta:
        model = Document
        fields = ("nom", "annee", "matiere", "fichier", "tags", "description")
        bbcode_field = "description"

    matiere = forms.ModelChoiceField(Matiere.objects.all(), widget=MatiereWidget())
    tags = TagField(Tag.objects.all(), widget=TagWidget(), required=False)
    fichier = DocumentFileField(widget=forms.FileInput())

    def __init__(self, *args, **kwargs):
        ecole = kwargs.pop("ecole")
        super(CreateDocumentForm, self).__init__(*args, **kwargs)
        self.fields["annee"].widget = forms.Select(
            choices=[("", "------")] + [(i, f"{i}/{i+1}") for i in range(date.today().year, 2000, -1)]
        )
        self.fields["matiere"].queryset = (
            Matiere.objects.filter(filiere__annee__ecole=ecole)
            .select_related("filiere__annee", "filiere")
            .order_by("filiere__annee__nom", "filiere__nom", "nom")
        )
        self.fields["tags"].queryset = Tag.objects.filter(ecole=ecole)

    def clean_fichier(self):
        fichier = self.cleaned_data["fichier"]
        taille_max = TAILLE_MAX_NOM_FICHIER - len(DOSSIER_STOCKAGE_FICHIERS) - 1

        if fichier and len(fichier.name) > taille_max:
            del self.cleaned_data["fichier"]
            raise ValidationError(f"Nom de fichier trop long (max {taille_max} caractères)")

        return fichier


class EditDocumentForm(CreateDocumentForm):
    fichier = DocumentFileField(widget=forms.FileInput(), required=False)


class CommentaireForm(forms.ModelForm):
    class Meta:
        model = Commentaire
        fields = ("message",)
